import argparse
import logging
import re

from aim import Run
from hydra import compose, initialize
from hydra.core.plugins import Plugins
from omegaconf import OmegaConf
from urartu.commands.command import Command
from urartu.utils.launcher import launch, launch_on_slurm
from urartu.utils.slurm import is_submitit_available
from urartu.utils.registry import Registry
from urartu.utils.hydra_plugin import UrartuPlugin

logging.basicConfig(level=logging.INFO, format="%(asctime)s - %(levelname)s - %(message)s")


def register_my_plugin() -> None:
    Plugins.instance().register(UrartuPlugin)


register_my_plugin()


@Command.register("launch")
class Launch(Command):
    """
    Launches an action from a specific module
    """

    def add_subparser(self, parser: argparse._SubParsersAction) -> argparse.ArgumentParser:
        description = """urartu: launcher"""
        available_modules = ", ".join(Registry.load_file_content().keys())
        subparser = parser.add_parser(
            self.name,
            description=description,
            help=f"Launch an action from a given module: {available_modules}",
        )

        subparser.add_argument("--name", type=str, help="name of the project/module")
        subparser.add_argument("module_args", nargs=argparse.REMAINDER, help="module arguments")

        subparser.set_defaults(fire=self._launch)

        return subparser

    def _launch(self, args: argparse.Namespace):
        module_name = re.sub(r"[^A-Za-z0-9]+", "", args.name)
        module_path = Registry.get_module_path_by_name(module_name)

        with initialize(version_base=None, config_path="../config"):
            cfg = compose(config_name="main", overrides=args.module_args)
        cfg = OmegaConf.create(OmegaConf.to_container(cfg, resolve=True, enum_to_str=True))

        aim_run = Run(
            repo=cfg.aim.repo,
            experiment=cfg.action_config.experiment_name,
            log_system_params=cfg.aim.log_system_params,
        )
        aim_run.set("cfg", cfg, strict=False)

        if cfg.slurm.use_slurm:
            assert is_submitit_available(), "Please 'pip install submitit' to schedule jobs on SLURM"

            launch_on_slurm(
                module=module_path,
                action_name=cfg.action_name,
                cfg=cfg,
                aim_run=aim_run,
            )
        else:
            launch(
                module=module_path,
                action_name=cfg.action_name,
                cfg=cfg,
                aim_run=aim_run,
            )

        if aim_run.active:
            aim_run.close()



from hydra.core.config_search_path import ConfigSearchPath
from hydra.plugins.search_path_plugin import SearchPathPlugin

from urartu.utils.registry import Registry
from urartu.utils.user import get_current_user
import logging

logger = logging.getLogger(__name__)

current_user = get_current_user()
registry_paths = Registry.get_module_paths()

class UrartuPlugin(SearchPathPlugin):
    def manipulate_search_path(self, search_path: ConfigSearchPath) -> None:
        if registry_paths:
            for registry_path in registry_paths:
                search_path.append(provider="urartu", path=f"file://{registry_path}/configs_{current_user}")
                search_path.append(provider="urartu", path=f"pkg://{registry_path}/configs_{current_user}")
                search_path.append(provider="urartu", path=f"file://{registry_path}/configs")
                search_path.append(provider="urartu", path=f"pkg://{registry_path}/configs")
        else:
            logger.warning("No registry paths found for UrartuPlugin.")


from typing import Tuple

import torch
from transformers import AutoModelForCausalLM, AutoTokenizer, pipeline

from urartu.common.device import Device
from urartu.common.model import Model
from urartu.utils.dtype import eval_dtype


class ModelPipeline(Model):
    def __init__(self, cfg) -> None:
        super().__init__(cfg)
        self._tokenizer = None

    @property
    def model(self):
        if self._model is None:
            clm_model = AutoModelForCausalLM.from_pretrained(
                self.cfg.name,
                cache_dir=self.cfg.cache_dir,
                device_map=Device.get_device(),
                torch_dtype=eval_dtype(self.cfg.dtype),
                token=self.cfg.api_token,
            )

            self._model = pipeline(
                "text-generation",
                model=clm_model,
                tokenizer=self.tokenizer,
                torch_dtype=eval_dtype(self.cfg.dtype),
                device_map=Device.get_device(),
                eos_token_id=self.tokenizer.eos_token_id,
            )
        return self._model

    @property
    def tokenizer(self):
        if self._tokenizer is None:
            self._tokenizer = AutoTokenizer.from_pretrained(self.cfg.name)
        return self._tokenizer

    def generate(self, prompt: str, generate_cfg):
        output = self.model(prompt, **generate_cfg)
        output_first_resp = output[0]["generated_text"]

        return output_first_resp



from typing import Any, Dict, List

from datasets import load_dataset
from pathlib import Path

from urartu.common.dataset import Dataset


class DatasetFromFile(Dataset):
    def __init__(self, cfg: List[Dict[str, Any]]) -> None:
        super().__init__(cfg)

    def _get_dataset(self):
        if "data_files" not in self.cfg:
            raise TypeError("Argument 'data_files' is missing")
        file_extension = Path(self.cfg.data_files).suffix
        if file_extension.startswith(".json"):
            file_format = "json"
        elif file_extension == ".txt":
            file_format = "text"
        else:
            raise KeyError(f"Failed to load data file '{file_extension}'")
        self.dataset = load_dataset(file_format, data_files=self.cfg.data_files)["train"]



from typing import Any, Dict, List

from datasets import Dataset as HFDataset

from urartu.common.dataset import Dataset


class DatasetFromDict(Dataset):
    def __init__(self, cfg: List[Dict[str, Any]]) -> None:
        super().__init__(cfg)

    def _get_dataset(self):
        if "data" not in self.cfg:
            raise TypeError("Argument 'data' is missing")

        self.dataset = HFDataset.from_dict(dict(self.cfg.data))



from typing import Any, Dict, List

from datasets import load_dataset

from urartu.common.dataset import Dataset


class DatasetFromHub(Dataset):
    def __init__(self, cfg: List[Dict[str, Any]]) -> None:
        super().__init__(cfg)

    def _get_dataset(self):
        if "name" not in self.cfg:
            raise TypeError("Argument 'name' is missing")
        self.dataset = load_dataset(
            self.cfg.name,
            self.cfg.get("subset"),
            split=self.cfg.get("split"),
        )
